{% extends "wagtailsnippets/snippets/type_index.html" %}
{% load i18n wagtailadmin_tags wagtail_airtable_tags %}


{% block content %}
    {% can_import_model model_opts.label as can_import_model %}
    <header class="nice-padding">
        <div class="row row-flush">
            <div class="left col6 header-title">
                <h1 class="icon icon-snippet">
                {% blocktrans with snippet_type_name_plural=model_opts.verbose_name_plural|capfirst %}Snippets <span>{{ snippet_type_name_plural }}</span>{% endblocktrans %}</h1>

                {% if is_searchable %}
                    <form class="col search-form" action="{% url 'wagtailsnippets:list' model_opts.app_label model_opts.model_name %}" method="get" novalidate>
                        <ul class="fields">
                            {% for field in search_form %}
                                {% include "wagtailadmin/shared/field_as_li.html" with field=field field_classes="field-small iconfield" input_classes="icon-search" %}
                            {% endfor %}
                            <li class="submit visuallyhidden"><input type="submit" value="Search" class="button" /></li>
                        </ul>
                    </form>
                {% endif %}
            </div>
            <div class="right col6">
                {% if can_delete_snippets %}
                    <a class="button bicolor icon icon-bin serious delete-button visuallyhidden" data-url="{% url 'wagtailsnippets:delete-multiple' model_opts.app_label model_opts.model_name %}?">{% blocktrans with snippet_type_name=model_opts.verbose_name_plural %}Delete {{ snippet_type_name }}{% endblocktrans %}</a>
                {% endif %}
                {% if can_add_snippet %}
                    <a href="{% url 'wagtailsnippets:add' model_opts.app_label model_opts.model_name %}" class="button bicolor icon icon-plus">{% blocktrans with snippet_type_name=model_opts.verbose_name %}Add {{ snippet_type_name }}{% endblocktrans %}</a>
                    {# TODO: figure out a way of saying "Add a/an [foo]" #}
                    {% if can_import_model %}
                      <form action="{% url 'airtable_import_listing' %}" method="post" style='display: inline-block;'>
                        {% csrf_token %}
                        <input type="hidden" name="model" value="{{ model_opts.label }}" />
                        <button type="submit" class="button bicolor icon icon-plus button-longrunning" data-clicked-text="Importing...">
                          <span class="icon icon-spinner"></span>
                          <em>{% blocktrans with snippet_type_name=model_opts.verbose_name_plural %}Import {{ snippet_type_name }}{% endblocktrans %}</em>
                        </button>
                      </form>
                    {% endif %}
                {% endif %}
            </div>
        </div>
    </header>

    <div class="nice-padding">
        <div id="snippet-results" class="snippets">
            {% include "wagtailsnippets/snippets/results.html" %}
        </div>
    </div>
{% endblock %}
