# Generated by Django 3.0.6 on 2020-05-15 18:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelNotUsed',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('airtable_record_id', models.CharField(blank=True, db_index=True, max_length=35)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
