# Generated by Django 3.0.6 on 2020-05-14 19:00

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0045_assign_unlock_grouppagepermission'),
    ]

    operations = [
        migrations.CreateModel(
            name='Advert',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('airtable_record_id', models.CharField(blank=True, db_index=True, max_length=35)),
                ('title', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True)),
                ('external_link', models.URLField(blank=True, max_length=500)),
                ('is_active', models.BooleanField(default=False)),
                ('rating', models.DecimalField(choices=[(1.0, '1'), (1.5, '1.5'), (2.0, '2'), (2.5, '2.5'), (3.0, '3'), (3.5, '3.5'), (4.0, '4'), (4.5, '4.5'), (5.0, '5')], decimal_places=1, max_digits=2, null=True)),
                ('long_description', wagtail.core.fields.RichTextField(blank=True, null=True)),
                ('points', models.IntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=100, unique=True)),
            ],
            options={
                'verbose_name': 'Advertisement',
                'verbose_name_plural': 'Advertisements',
            },
        ),
        migrations.CreateModel(
            name='SimplePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('intro', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]
